package sun.security.krb5;

import java.util.StringTokenizer;
import sun.security.krb5.internal.Krb5;
import javax.naming.Context;
import java.util.Arrays;
import javax.naming.directory.Attribute;
import java.security.PrivilegedActionException;
import javax.naming.NamingException;
import java.security.AccessController;
import java.net.SocketPermission;
import java.security.Permission;
import javax.naming.directory.Attributes;
import javax.naming.spi.NamingManager;
import java.util.Hashtable;
import javax.naming.directory.DirContext;
import java.util.Random;

class KrbServiceLocator
{
    private static final String SRV_RR = "SRV";
    private static final String[] SRV_RR_ATTR;
    private static final String SRV_TXT = "TXT";
    private static final String[] SRV_TXT_ATTR;
    private static final Random random;
    private static final boolean DEBUG;
    
    private KrbServiceLocator() {
    }
    
    static String[] getKerberosService(String str) {
        new StringBuilder().append("dns:///_kerberos.").append(str).toString();
        String[] array = null;
        try {
            if (!(NamingManager.getURLContext("dns", new Hashtable(0)) instanceof DirContext)) {
                return null;
            }
            Attributes attributes;
            try {
                attributes = (Attributes)AccessController.doPrivileged(()/*  */ -> ((DirContext)var_3_23).getAttributes(var_1_13, KrbServiceLocator.SRV_TXT_ATTR), null, new Permission[] { new SocketPermission("*", "connect,accept") });
            }
            catch (PrivilegedActionException ex) {
                throw (NamingException)ex.getCause();
            }
            Attribute value;
            if (attributes != null && (value = attributes.get("TXT")) != null) {
                int size = value.size();
                String[] array2 = new String[size];
                int i = 0;
                int n = 0;
                while (i < size) {
                    try {
                        array2[n] = (String)value.get(i);
                        ++n;
                    }
                    catch (Exception ex2) {}
                    ++i;
                }
                int n2 = n;
                if (n2 < size) {
                    String[] array3 = new String[n2];
                    System.arraycopy(array2, 0, array3, 0, n2);
                    array = array3;
                }
                else {
                    array = array2;
                }
            }
        }
        catch (NamingException ex3) {}
        return array;
    }
    
    static String[] getKerberosService(String str, String str2) {
        new StringBuilder().append("dns:///_kerberos.").append(str2).append(".").append(str).toString();
        String[] hostports = null;
        try {
            if (!(NamingManager.getURLContext("dns", new Hashtable(0)) instanceof DirContext)) {
                return null;
            }
            Attributes attributes;
            try {
                attributes = (Attributes)AccessController.doPrivileged(()/*  */ -> ((DirContext)var_4_2C).getAttributes(var_2_1C, KrbServiceLocator.SRV_RR_ATTR), null, new Permission[] { new SocketPermission("*", "connect,accept") });
            }
            catch (PrivilegedActionException ex) {
                throw (NamingException)ex.getCause();
            }
            Attribute value;
            if (attributes != null && (value = attributes.get("SRV")) != null) {
                int size = value.size();
                KrbServiceLocator.SrvRecord[] a = new KrbServiceLocator.SrvRecord[size];
                int i = 0;
                int n = 0;
                while (i < size) {
                    try {
                        a[n] = new KrbServiceLocator.SrvRecord((String)value.get(i));
                        ++n;
                    }
                    catch (Exception ex2) {}
                    ++i;
                }
                int n2 = n;
                if (n2 < size) {
                    KrbServiceLocator.SrvRecord[] array = new KrbServiceLocator.SrvRecord[n2];
                    System.arraycopy(a, 0, array, 0, n2);
                    a = array;
                }
                if (n2 > 1) {
                    Arrays.sort(a);
                }
                hostports = extractHostports(a);
            }
        }
        catch (NamingException ex3) {}
        return hostports;
    }
    
    private static String[] extractHostports(KrbServiceLocator.SrvRecord[] array) {
        String[] array2 = null;
        int n = 0;
        for (int i = 0; i < array.length; ++i) {
            if (array2 == null) {
                array2 = new String[array.length];
            }
            int n2 = i;
            while (i < array.length - 1 && array[i].priority == array[i + 1].priority) {
                ++i;
            }
            int n3 = i;
            for (int n4 = n3 - n2 + 1, j = 0; j < n4; ++j) {
                array2[n++] = selectHostport(array, n2, n3);
            }
        }
        return array2;
    }
    
    private static String selectHostport(KrbServiceLocator.SrvRecord[] array, int i, int n) {
        if (i == n) {
            return array[i].hostport;
        }
        int sum = 0;
        for (; i <= n; ++i) {
            if (array[i] != null) {
                sum += array[i].weight;
                array[i].sum = sum;
            }
        }
        String hostport = null;
        int n2 = (sum == 0) ? 0 : KrbServiceLocator.random.nextInt(sum + 1);
        while (i <= n) {
            if (array[i] != null && array[i].sum >= n2) {
                hostport = array[i].hostport;
                array[i] = null;
                break;
            }
            ++i;
        }
        return hostport;
    }
    
    static {
        SRV_RR_ATTR = new String[] { "SRV" };
        SRV_TXT_ATTR = new String[] { "TXT" };
        random = new Random();
        DEBUG = Krb5.DEBUG;
    }
    
    static class SrvRecord implements Comparable<KrbServiceLocator.SrvRecord>
    {
        int priority;
        int weight;
        int sum;
        String hostport;
        
        SrvRecord(String str) throws Exception {
            StringTokenizer stringTokenizer = new StringTokenizer(str, " ");
            if (stringTokenizer.countTokens() == 4) {
                this.priority = Integer.parseInt(stringTokenizer.nextToken());
                this.weight = Integer.parseInt(stringTokenizer.nextToken());
                this.hostport = stringTokenizer.nextToken() + ":" + stringTokenizer.nextToken();
                return;
            }
            throw new IllegalArgumentException();
        }
        
        @Override
        public int compareTo(KrbServiceLocator.SrvRecord srvRecord) {
            if (this.priority > srvRecord.priority) {
                return 1;
            }
            if (this.priority < srvRecord.priority) {
                return -1;
            }
            if (this.weight == 0 && srvRecord.weight != 0) {
                return -1;
            }
            if (this.weight != 0 && srvRecord.weight == 0) {
                return 1;
            }
            return 0;
        }
    }
}
